package kr.co.vividnext.sodalive.admin.content.curation.tag

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.curation.tag.ContentHashTagCurationItem
import kr.co.vividnext.sodalive.content.main.curation.tag.QContentHashTagCuration.contentHashTagCuration
import kr.co.vividnext.sodalive.content.main.curation.tag.QContentHashTagCurationItem.contentHashTagCurationItem
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AdminContentHashTagCurationItemRepository :
    JpaRepository<ContentHashTagCurationItem, Long>,
    AdminContentHashTagCurationItemQueryRepository

interface AdminContentHashTagCurationItemQueryRepository {
    fun getContentHashTagCurationItemList(curationId: Long): List<GetAdminHashTagCurationItemResponse>
    fun findByCurationIdAndContentId(curationId: Long, contentId: Long?): ContentHashTagCurationItem?
    fun findByCurationIdAndItemId(curationId: Long, itemId: Long): ContentHashTagCurationItem?
}

class AdminContentHashTagCurationItemQueryRepositoryImpl(
    val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) : AdminContentHashTagCurationItemQueryRepository {
    override fun getContentHashTagCurationItemList(curationId: Long): List<GetAdminHashTagCurationItemResponse> {
        return queryFactory
            .select(
                QGetAdminHashTagCurationItemResponse(
                    contentHashTagCurationItem.id,
                    audioContent.title,
                    audioContent.detail,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.member.nickname.coalesce(""),
                    audioContent.isAdult
                )
            )
            .from(contentHashTagCurationItem)
            .innerJoin(contentHashTagCurationItem.curation, contentHashTagCuration)
            .innerJoin(contentHashTagCurationItem.content, audioContent)
            .where(
                contentHashTagCuration.id.eq(curationId),
                contentHashTagCurationItem.isActive.isTrue,
                audioContent.isActive.isTrue
            )
            .orderBy(contentHashTagCurationItem.orders.asc())
            .fetch()
    }

    override fun findByCurationIdAndContentId(curationId: Long, contentId: Long?): ContentHashTagCurationItem? {
        return queryFactory
            .selectFrom(contentHashTagCurationItem)
            .innerJoin(contentHashTagCurationItem.curation, contentHashTagCuration)
            .innerJoin(contentHashTagCurationItem.content, audioContent)
            .where(
                contentHashTagCuration.id.eq(curationId),
                audioContent.id.eq(contentId)
            )
            .fetchFirst()
    }

    override fun findByCurationIdAndItemId(curationId: Long, itemId: Long): ContentHashTagCurationItem? {
        return queryFactory.selectFrom(contentHashTagCurationItem)
            .innerJoin(contentHashTagCurationItem.curation, contentHashTagCuration)
            .where(
                contentHashTagCuration.id.eq(curationId),
                contentHashTagCurationItem.id.eq(itemId)
            )
            .fetchFirst()
    }
}
