package kr.co.vividnext.sodalive.admin.event.banner

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/admin/event/banner")
@PreAuthorize("hasRole('ADMIN')")
class AdminEventBannerController(private val service: AdminEventBannerService) {
    @PostMapping
    fun createEventBanner(
        @RequestParam("thumbnail") thumbnail: MultipartFile,
        @RequestParam(value = "detail", required = false) detail: MultipartFile? = null,
        @RequestParam(value = "popup", required = false) popup: MultipartFile? = null,
        @RequestParam(value = "link", required = false) link: String? = null,
        @RequestParam(value = "title", required = false) title: String? = null,
        @RequestParam(value = "isAdult", required = false) isAdult: Boolean? = null,
        @RequestParam(value = "isPopup") isPopup: Boolean,
        @RequestParam(value = "startDate") startDate: String,
        @RequestParam(value = "endDate") endDate: String
    ) = ApiResponse.ok(
        service.save(thumbnail, detail, popup, link, title, isAdult, isPopup, startDate, endDate),
        "등록되었습니다."
    )

    @PutMapping
    fun updateEvent(
        @RequestParam(value = "id") id: Long,
        @RequestParam(value = "thumbnail", required = false) thumbnail: MultipartFile? = null,
        @RequestParam(value = "detail", required = false) detail: MultipartFile? = null,
        @RequestParam(value = "popup", required = false) popup: MultipartFile? = null,
        @RequestParam(value = "link", required = false) link: String? = null,
        @RequestParam(value = "title", required = false) title: String? = null,
        @RequestParam(value = "isAdult", required = false) isAdult: Boolean? = null,
        @RequestParam(value = "isPopup", required = false) isPopup: Boolean? = null,
        @RequestParam(value = "startDate", required = false) startDate: String? = null,
        @RequestParam(value = "endDate", required = false) endDate: String? = null
    ) = ApiResponse.ok(
        service.update(id, thumbnail, detail, popup, link, title, isAdult, isPopup, startDate, endDate),
        "수정되었습니다."
    )

    @DeleteMapping("/{id}")
    fun deleteEvent(@PathVariable id: Long) = ApiResponse.ok(service.delete(id), "삭제되었습니다.")

    @GetMapping
    fun getEventList() = ApiResponse.ok(service.getEventList())
}
