package kr.co.vividnext.sodalive.admin.point

import kr.co.vividnext.sodalive.point.PointRewardPolicy
import kr.co.vividnext.sodalive.useraction.ActionType
import kr.co.vividnext.sodalive.useraction.PolicyType
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

data class CreatePointRewardPolicyRequest(
    val title: String,
    val policyType: PolicyType,
    val actionType: ActionType,
    val threshold: Int,
    val availableCount: Int,
    val pointAmount: Int,
    val startDate: String,
    val endDate: String
) {
    fun toEntity(): PointRewardPolicy {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")

        return PointRewardPolicy(
            title = title,
            policyType = policyType,
            actionType = actionType,
            threshold = threshold,
            availableCount = availableCount,
            pointAmount = pointAmount,
            startDate = LocalDateTime.parse(startDate, dateTimeFormatter)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime(),
            endDate = if (endDate.isNotBlank()) {
                LocalDateTime.parse(endDate, dateTimeFormatter).withSecond(59)
                    .atZone(ZoneId.of("Asia/Seoul"))
                    .withZoneSameInstant(ZoneId.of("UTC"))
                    .toLocalDateTime()
            } else {
                null
            },
            isActive = true
        )
    }
}
