package kr.co.vividnext.sodalive.admin.point

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.point.PointRewardPolicy
import kr.co.vividnext.sodalive.point.QPointRewardPolicy.pointRewardPolicy
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface PointPolicyRepository : JpaRepository<PointRewardPolicy, Long>, PointPolicyQueryRepository

interface PointPolicyQueryRepository {
    fun getTotalCount(): Int
    fun getAll(offset: Long, limit: Long): List<GetPointRewardPolicyListItem>
}

class PointPolicyQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : PointPolicyQueryRepository {
    override fun getTotalCount(): Int {
        return queryFactory
            .select(pointRewardPolicy.id)
            .from(pointRewardPolicy)
            .fetch()
            .size
    }

    override fun getAll(offset: Long, limit: Long): List<GetPointRewardPolicyListItem> {
        return queryFactory
            .select(
                QGetPointRewardPolicyListItem(
                    pointRewardPolicy.id,
                    pointRewardPolicy.title,
                    pointRewardPolicy.policyType,
                    pointRewardPolicy.actionType,
                    pointRewardPolicy.threshold,
                    pointRewardPolicy.availableCount,
                    pointRewardPolicy.pointAmount,
                    getFormattedDate(pointRewardPolicy.startDate),
                    getFormattedDate(pointRewardPolicy.endDate),
                    pointRewardPolicy.isActive
                )
            )
            .from(pointRewardPolicy)
            .orderBy(pointRewardPolicy.isActive.desc(), pointRewardPolicy.startDate.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "COALESCE(DATE_FORMAT(CONVERT_TZ({0}, 'UTC', 'Asia/Seoul'), '%Y-%m-%d %H:%i'), '')",
            dateTimePath
        )
    }
}
