package kr.co.vividnext.sodalive.member.nickname

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.stereotype.Service
import kotlin.random.Random

@Service
class NicknameGenerateService(private val repository: MemberRepository) {
    private val adjectives = listOf(
        "따뜻한", "은은한", "고요한", "푸른", "맑은", "강한", "평온한", "깊은", "고독한",
        "거친", "빛바랜", "차가운", "꿈꾸는", "숨겨진", "고귀한", "깨어난", "끝없는", "청명한",
        "어두운", "희미한", "선명한", "눈부신", "불타는", "차분한", "아련한", "선선한", "상쾌한", "온화한",
        "포근한", "황금빛", "청량한", "시원한", "서늘한", "우아한", "단단한", "투명한", "가벼운", "조용한",
        "화려한", "찬란한", "순수한", "흐릿한", "고결한", "달콤한", "무한한", "아득한", "화사한", "평안한",
        "웅장한", "황홀한", "빛나는", "쓸쓸한", "청순한", "흐르는", "미묘한", "그윽한", "몽롱한", "청아한",
        "섬세한", "촉촉한", "강렬한", "싱싱한", "부드런", "아늑한", "매운듯", "고운듯", "느린듯", "밝은듯",
        "짧은듯", "달큰한", "깊은듯", "기쁜듯", "쌀쌀한", "무거운", "연한듯", "편안한", "깨끗한", "말끔한",
        "뽀얀듯", "푸르른", "붉은듯", "노오란", "무딘듯", "살짝한", "상큼한", "시큰한", "고소한", "나긋한",
        "화끈한", "중후한", "정겨운", "날렵한", "기묘한", "참신한", "담백한", "퉁명한", "꾸밈없", "소박한",
        "뾰족한", "무심한", "도도한", "따끔한", "무난한", "단호한", "냉정한", "따스한", "유연한", "묵직한",
        "나른한", "몽환적", "정돈된", "쾌활한", "날카론", "묘한듯", "예쁜듯", "뽀얗게", "다정한", "푸근한",
        "애틋한", "낭만적", "건조한", "훈훈한", "섹시한", "정적인", "유쾌한", "멍한듯", "혼란한", "상냥한",
        "뚜렷한", "신비한", "허전한", "그리운", "들뜬듯", "절실한", "반듯한", "반가운", "새하얀", "흐린듯",
        "엄숙한", "깊잖은", "산뜻한", "낯선듯"
    )

    private val nouns = listOf(
        "소리", "울림", "공명", "음색", "감성", "리듬", "바람", "늑대", "태양", "대지",
        "강", "하늘", "불꽃", "별빛", "나무", "산", "달빛", "폭풍", "눈", "밤",
        "노을", "물결", "노래", "파도", "구름", "사슴", "신비", "영혼", "선율", "평원",
        "빛", "고래", "모래", "사자", "표범", "여우", "곰", "수달", "판다", "들소",
        "까치", "매", "솔개", "물총새", "철새", "황새", "은어", "붕어", "산양", "담비",
        "설표", "물개", "자라", "나비", "노루", "해마", "백조", "청어", "호수", "샘물",
        "쿼카", "상어", "무드", "나노", "루프", "네온", "모아", "아토", "플로", "루미",
        "도트", "비트", "토브", "온기", "클리", "위드", "제로", "베이", "미오", "시그",
        "쿠나", "오로", "폴라", "바움", "포잇", "누아", "오브", "파인", "조이", "아뜰",
        "티노", "소마", "하루", "밀크", "아린", "토로", "벨로", "위시", "뮤즈", "노블",
        "카노", "미카", "하라", "엘로", "피오", "라임", "노이", "루다", "이브", "마리",
        "블루", "시온", "레아", "도르", "하노", "네리", "키노", "쿠키", "라노", "수이",
        "우노", "파루", "크리", "포유", "코코", "아라", "토리", "누리", "보노", "페어",
        "리아", "모리", "세리", "리브", "헤이", "모카", "아이", "르네", "이로", "미노",
        "다라", "노바", "디노", "오미", "카라", "니아", "루아", "네오", "하이", "레인",
        "피카", "유카", "제니", "이든", "라비", "아벨", "솔라", "쿠로", "시라", "리코"
    )

    private fun generateRandomNickname(): String {
        val formatType = Random.nextInt(3)
        return when (formatType) {
            0 -> "${adjectives.random()}${nouns.random()}"
            1 -> "${nouns.random()}의${nouns.random()}"
            else -> "${adjectives.random()}${nouns.random()}의${nouns.random()}"
        }
    }

    private fun generateNonConflictingNickname(usedNicknames: Set<String>): String {
        val usedNicknameSet = HashSet(usedNicknames) // 해시셋으로 변환 (O(1) 조회 가능)
        val availableNumbers = (1000..9999).shuffled()

        for (num in availableNumbers) { // 숫자를 먼저 결정 (무작위)
            for (adj in adjectives.shuffled()) { // 형용사 순서 랜덤화
                for (noun in nouns.shuffled()) { // 명사 순서 랜덤화
                    val candidate = "$adj$noun$num"
                    if (!usedNicknameSet.contains(candidate)) {
                        return candidate
                    }
                }
            }
        }
        throw SodaException("회원가입을 하지 못했습니다.\n다시 시도해 주세요.")
    }

    fun generateUniqueNickname(): String {
        repeat(5) {
            val candidates = (1..10).map { generateRandomNickname() }
            val available = candidates.firstOrNull { !repository.existsByNickname(it) }
            if (available != null) return available
        }

        return generateNonConflictingNickname(repository.findNicknamesWithPrefix("").toSet())
    }
}
