package kr.co.vividnext.sodalive.member.social.google

import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier
import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport
import com.google.api.client.json.gson.GsonFactory
import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service

@Service
class GoogleService(
    @Value("\${google.web-client-id}")
    private val googleWebClientId: String
) {
    private val transport = GoogleNetHttpTransport.newTrustedTransport()
    private val jsonFactory = GsonFactory.getDefaultInstance()
    private val clientIds = listOf(googleWebClientId)

    fun getUserInfo(idToken: String): GoogleUserInfo? {
        val verifier = GoogleIdTokenVerifier.Builder(transport, jsonFactory)
            .setAudience(clientIds)
            .setIssuers(listOf("accounts.google.com", "https://accounts.google.com"))
            .build()

        return try {
            val token = verifier.verify(idToken)

            if (token != null) {
                val payload = token.payload
                val email = payload.email ?: throw SodaException("이메일 제공에 동의하셔야 서비스 이용이 가능합니다.")

                GoogleUserInfo(
                    sub = payload.subject,
                    email = email,
                    name = payload["name"] as? String
                )
            } else {
                null
            }
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    }
}
