package kr.co.vividnext.sodalive.report

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.explorer.profile.CreatorCheersRepository
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.CreatorCommunityRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
@Transactional(readOnly = true)
class ReportService(
    private val repository: ReportRepository,
    private val memberRepository: MemberRepository,
    private val cheersRepository: CreatorCheersRepository,
    private val creatorCommunityRepository: CreatorCommunityRepository
) {
    @Transactional
    fun save(member: Member, request: ReportRequest) {
        if (conditionAllIsNull(request, isNull = true) || conditionAllIsNull(request, isNull = false)) {
            throw SodaException("신고가 접수되었습니다.")
        }

        val reportedAccount = if (request.reportedMemberId != null) {
            memberRepository.findByIdOrNull(request.reportedMemberId)
                ?: throw SodaException("신고가 접수되었습니다.")
        } else {
            null
        }

        val cheers = if (request.cheersId != null) {
            cheersRepository.findByIdOrNull(request.cheersId)
                ?: throw SodaException("신고가 접수되었습니다.")
        } else {
            null
        }

        val communityPost = if (request.communityPostId != null) {
            creatorCommunityRepository.findByIdOrNull(request.communityPostId)
                ?: throw SodaException("신고가 접수되었습니다.")
        } else {
            null
        }

        val report = Report(type = request.type, reason = request.reason)
        report.member = member
        report.reportedAccount = reportedAccount
        report.cheers = cheers
        report.communityPost = communityPost
        repository.save(report)
    }

    private fun conditionAllIsNull(request: ReportRequest, isNull: Boolean): Boolean {
        return if (isNull) {
            request.reportedMemberId == null &&
                request.cheersId == null &&
                request.audioContentId == null &&
                request.communityPostId == null
        } else {
            request.reportedMemberId != null &&
                request.cheersId != null &&
                request.audioContentId != null &&
                request.communityPostId != null
        }
    }
}
